@@ -36,34 +36,42 @@ impl P256PublicKey {
 }
 
 #[doc(hidden)]
-pub struct P256KeyPair(ecdsa::SigningKey);
+pub struct P256KeyPair {
+    p256_sk: ecdsa::SigningKey,
+    metadata: Option<KeyMetadata>,
+}
 
 impl AsRef<ecdsa::SigningKey> for P256KeyPair {
     fn as_ref(&self) -> &ecdsa::SigningKey {
-        &self.0
+        &self.p256_sk
     }
 }
 
 impl P256KeyPair {
     pub fn from_bytes(raw: &[u8]) -> Result<Self, Error> {
-        let p256_key_pair =
-            ecdsa::SigningKey::from_bytes(raw).map_err(|_| JWTError::InvalidKeyPair)?;
-        Ok(P256KeyPair(p256_key_pair))
+        let p256_sk = ecdsa::SigningKey::from_bytes(raw).map_err(|_| JWTError::InvalidKeyPair)?;
+        Ok(P256KeyPair {
+            p256_sk,
+            metadata: None,
+        })
     }
 
     pub fn to_bytes(&self) -> Vec<u8> {
-        self.0.to_bytes().to_vec()
+        self.p256_sk.to_bytes().to_vec()
     }
 
     pub fn public_key(&self) -> P256PublicKey {
-        let p256_pk = self.0.verifying_key();
+        let p256_pk = self.p256_sk.verifying_key();
         P256PublicKey(p256_pk)
     }
 
     pub fn generate() -> Self {
         let rng = rand::thread_rng();
         let p256_sk = ecdsa::SigningKey::random(rng);
-        P256KeyPair(p256_sk)
+        P256KeyPair {
+            p256_sk,
+            metadata: None,
+        }
     }
 }
 
@@ -71,16 +79,15 @@ pub trait ECDSAP256KeyPairLike {
     fn jwt_alg_name() -> &'static str;
     fn key_pair(&self) -> &P256KeyPair;
     fn key_id(&self) -> &Option<String>;
+    fn metadata(&self) -> &Option<KeyMetadata>;
+    fn attach_metadata(&mut self, metadata: KeyMetadata) -> Result<(), Error>;
 
     fn sign<CustomClaims: Serialize + DeserializeOwned>(
         &self,
         claims: JWTClaims<CustomClaims>,
     ) -> Result<String, Error> {
-        let jwt_header = JWTHeader {
-            algorithm: Self::jwt_alg_name().to_string(),
-            key_id: self.key_id().clone(),
-            ..Default::default()
-        };
+        let jwt_header = JWTHeader::new(Self::jwt_alg_name().to_string(), self.key_id().clone())
+            .with_metadata(self.metadata());
         Token::build(&jwt_header, claims, |authenticated| {
             let mut digest = hmac_sha256::Hash::new();
             digest.update(authenticated.as_bytes());
@@ -155,6 +162,15 @@ impl ECDSAP256KeyPairLike for ES256KeyPair {
     fn key_id(&self) -> &Option<String> {
         &self.key_id
     }
+
+    fn metadata(&self) -> &Option<KeyMetadata> {
+        &self.key_pair.metadata
+    }
+
+    fn attach_metadata(&mut self, metadata: KeyMetadata) -> Result<(), Error> {
+        self.key_pair.metadata = Some(metadata);
+        Ok(())
+    }
 }
 
 impl ES256KeyPair {
