@@ -45,68 +45,76 @@ impl K256PublicKey {
 }
 
 #[doc(hidden)]
-pub struct K256KeyPair(ecdsa::SigningKey);
+pub struct K256KeyPair {
+    k256_sk: ecdsa::SigningKey,
+    metadata: Option<KeyMetadata>,
+}
 
 impl AsRef<ecdsa::SigningKey> for K256KeyPair {
     fn as_ref(&self) -> &ecdsa::SigningKey {
-        &self.0
+        &self.k256_sk
     }
 }
 
 impl K256KeyPair {
     pub fn from_bytes(raw: &[u8]) -> Result<Self, Error> {
-        let k256_key_pair =
-            ecdsa::SigningKey::from_bytes(raw).map_err(|_| JWTError::InvalidKeyPair)?;
-        Ok(K256KeyPair(k256_key_pair))
+        let k256_sk = ecdsa::SigningKey::from_bytes(raw).map_err(|_| JWTError::InvalidKeyPair)?;
+        Ok(K256KeyPair {
+            k256_sk,
+            metadata: None,
+        })
     }
 
     pub fn from_der(der: &[u8]) -> Result<Self, Error> {
-        let k256_key_pair =
+        let k256_sk =
             ecdsa::SigningKey::from_pkcs8_der(der).map_err(|_| JWTError::InvalidKeyPair)?;
-        Ok(K256KeyPair(k256_key_pair))
+        Ok(K256KeyPair {
+            k256_sk,
+            metadata: None,
+        })
     }
 
     pub fn from_pem(pem: &str) -> Result<Self, Error> {
-        let k256_key_pair =
+        let k256_sk =
             ecdsa::SigningKey::from_pkcs8_pem(pem).map_err(|_| JWTError::InvalidKeyPair)?;
-        Ok(K256KeyPair(k256_key_pair))
+        Ok(K256KeyPair {
+            k256_sk,
+            metadata: None,
+        })
     }
 
     pub fn to_bytes(&self) -> Vec<u8> {
-        self.0.to_bytes().to_vec()
+        self.k256_sk.to_bytes().to_vec()
     }
 
     pub fn public_key(&self) -> K256PublicKey {
-        let k256_pk = self.0.verifying_key();
+        let k256_pk = self.k256_sk.verifying_key();
         K256PublicKey(k256_pk)
     }
 
     pub fn generate() -> Self {
         let rng = rand::thread_rng();
         let k256_sk = ecdsa::SigningKey::random(rng);
-        K256KeyPair(k256_sk)
+        K256KeyPair {
+            k256_sk,
+            metadata: None,
+        }
     }
 }
 
 pub trait ECDSAP256kKeyPairLike {
-    #[doc(hidden)]
     fn jwt_alg_name() -> &'static str;
-
-    #[doc(hidden)]
     fn key_pair(&self) -> &K256KeyPair;
-
-    #[doc(hidden)]
     fn key_id(&self) -> &Option<String>;
+    fn metadata(&self) -> &Option<KeyMetadata>;
+    fn attach_metadata(&mut self, metadata: KeyMetadata) -> Result<(), Error>;
 
     fn sign<CustomClaims: Serialize + DeserializeOwned>(
         &self,
         claims: JWTClaims<CustomClaims>,
     ) -> Result<String, Error> {
-        let jwt_header = JWTHeader {
-            algorithm: Self::jwt_alg_name().to_string(),
-            key_id: self.key_id().clone(),
-            ..Default::default()
-        };
+        let jwt_header = JWTHeader::new(Self::jwt_alg_name().to_string(), self.key_id().clone())
+            .with_metadata(self.metadata());
         Token::build(&jwt_header, claims, |authenticated| {
             let mut digest = hmac_sha256::Hash::new();
             digest.update(authenticated.as_bytes());
@@ -181,6 +189,15 @@ impl ECDSAP256kKeyPairLike for ES256kKeyPair {
     fn key_id(&self) -> &Option<String> {
         &self.key_id
     }
+
+    fn metadata(&self) -> &Option<KeyMetadata> {
+        &self.key_pair.metadata
+    }
+
+    fn attach_metadata(&mut self, metadata: KeyMetadata) -> Result<(), Error> {
+        self.key_pair.metadata = Some(metadata);
+        Ok(())
+    }
 }
 
 impl ES256kKeyPair {
